/* hw/s3c2410x.c
 *
 * Samsung S3C2410X emulation
 *
 * Copyright 2006 Daniel Silverstone and Vincent Sanders
 *
 * This file is under the terms of the GNU General Public
 * License Version 2
 */

#include "vl.h"
#include "s3c2410x.h"

/* Initialise a Samsung S3C2410X and all its internal peripherals */
void
s3c2410x_init(int ram_size)
{
  CPUState *env;
  /* Prepare the ARM 920T */
  env = cpu_init();
  cpu_arm_set_model(env, ARM_CPUID_ARM920T);
  /* S3C2410X memory is always at the same physical location */
  S3C2410X_DBF("RAM\n");
  cpu_register_physical_memory(CPU_S3C2410X_RAM, ram_size, IO_MEM_RAM);
  /* We get a memc */
  S3C2410X_DBF("memc\n");
  s3c2410x_memc_init();
  /* The clock control allows for idling the CPU */
  s3c2410x_clkcon_init(env);
  /* And some GPIO */
  S3C2410X_DBF("gpio\n");
  s3c2410x_gpio_init(); 
  /* RTC for time */
  s3c2410x_rtc_init();
  /* And some interrupts (coo) */
  S3C2410X_DBF("interrupts\n");
  s3c2410x_irq_init(env);
  /* And some IIC */
  s3c2410x_iic_init();
  /* And some timers */
  s3c2410x_timers_init();
  /* Serial ports */
  S3C2410X_DBF("serial\n");
  s3c2410x_serial_init(0);
  s3c2410x_serial_init(1);
  s3c2410x_serial_init(2);
  /* S3C2410X SRAM */
  cpu_register_physical_memory(CPU_S3C2410X_SRAM, 4096, ram_size | IO_MEM_RAM);
  /* A two port OHCI controller */
  usb_ohci_mmap_init(2, s3c2410x_irq_raise_interrupt_sif, 0, 26, CPU_S3C2410X_OHCI, 0x5C);
}

