/* hw/s3c2410x_iic.c
 *
 * Samsung S3C2410X emulation
 *
 * Copyright 2006 Daniel Silverstone and Vincent Sanders
 *
 * This file is under the terms of the GNU General Public
 * License Version 2
 */

#include "vl.h"
#include "s3c2410x.h"

static uint32_t s3c2410x_iic_reg[4];

static void
s3c2410x_iic_write_f(void *opaque_ignored, target_phys_addr_t addr_, uint32_t value)
{
  int addr = (addr_ & 12) >> 2;
  if (addr < 0 || addr > 3) addr = 3;
  S3C2410X_DBF("Write IIC[%02x] = %08x\n",addr<<2, value);
  s3c2410x_iic_reg[addr] = value;
  if( addr == 1 ) {
    /* A write to IICSTAT */
    if( (value & (1<<5)) && ((value & (3<<6)) == (3<<6)) ) {
      /* When we're Master TX and told to start, we hiccough inappropriately */
      S3C2410X_DBF("Oooh, IIC write to fail, yum!\n");
      value &= ~(1<<5); /* Not running */
      value |= (1<<3);  /* Arbitration failure */
      s3c2410x_iic_reg[1] = value;
      s3c2410x_iic_reg[0] |= (1<<4); /* Claim an interrupt */
      s3c2410x_irq_raise_interrupt(27);
    }
  }
}

static uint32_t
s3c2410x_iic_read_f(void *opaque_ignored, target_phys_addr_t addr_)
{
  int addr = (addr_ & 12) >> 2;
  if (addr < 0 || addr > 3) addr = 3;
//  S3C2410X_DBF("Read IIC[%02x]\n",addr<<2);
  return s3c2410x_iic_reg[addr];
}


static CPUReadMemoryFunc *s3c2410x_iic_read[] = {
  &s3c2410x_iic_read_f,
  &s3c2410x_iic_read_f,
  &s3c2410x_iic_read_f,
};

static CPUWriteMemoryFunc *s3c2410x_iic_write[] = {
  &s3c2410x_iic_write_f,
  &s3c2410x_iic_write_f,
  &s3c2410x_iic_write_f,
};


void
s3c2410x_iic_init(void)
{
  /* Samsung S3C2410X IIC magic badger.
   *
   * The only really magic thing we do currently is the ID register
   */
  int tag = cpu_register_io_memory(0, s3c2410x_iic_read, s3c2410x_iic_write, 0);
  cpu_register_physical_memory(CPU_S3C2410X_IIC_BASE, 4*4, tag);
  s3c2410x_iic_reg[0] = 0;
  s3c2410x_iic_reg[1] = 0;
  s3c2410x_iic_reg[2] = 0;
  s3c2410x_iic_reg[3] = 0;
  
}
