/* hw/s3c2410x_memc.c
 *
 * Samsung S3C2410X emulation
 *
 * Copyright 2006 Daniel Silverstone and Vincent Sanders
 *
 * This file is under the terms of the GNU General Public
 * License Version 2
 */

#include "vl.h"
#include "s3c2410x.h"

static uint32_t s3c2410x_memc_reg[13];

static void
s3c2410x_memc_write_f(void *opaque_ignored, target_phys_addr_t addr_, uint32_t value)
{
  int addr = (addr_ & 0x3f) >> 2;
  if (addr < 0 || addr > 12) addr = 12;
  /* S3C2410X_DBF("Write MEMC[%d] = %08x\n", addr, value); */
  s3c2410x_memc_reg[addr] = value;
}

static uint32_t
s3c2410x_memc_read_f(void *opaque_ignored, target_phys_addr_t addr_)
{
  int addr = (addr_ & 0x3f) >> 2;
  if (addr < 0 || addr > 12) addr = 12;
  /* S3C2410X_DBF("Read MEMC[%d]\n", addr); */
  return s3c2410x_memc_reg[addr];
}

static CPUReadMemoryFunc *s3c2410x_memc_read[] = {
  &s3c2410x_memc_read_f,
  &s3c2410x_memc_read_f,
  &s3c2410x_memc_read_f,
};

static CPUWriteMemoryFunc *s3c2410x_memc_write[] = {
  &s3c2410x_memc_write_f,
  &s3c2410x_memc_write_f,
  &s3c2410x_memc_write_f,
};


void
s3c2410x_memc_init(void)
{
  /* Memory controller is all about SDRAM control, thus we just drop
   * some RAM in there and assume life will be ok.
   * It is 13 registers, each 4 bytes.
   *
   * However because of the bizarro mechanisms in qemu for RAM, we do
   * it as MMIO which means a tag and then a memory range.
   */
  int tag;
  tag = cpu_register_io_memory(0, s3c2410x_memc_read, s3c2410x_memc_write, 0);
  cpu_register_physical_memory(CPU_S3C2410X_BWSCON, 13*4, tag);
  
  for(tag = 0; tag < 13; tag++) s3c2410x_memc_reg[tag] = 0;
  
}
