/* hw/simtecbast_ide.c
 *
 * MMIO interface to IDE on Simtec's BAST
 *
 * Copyright Daniel Silverstone and Vincent Sanders
 *
 * This file is under the terms of the GNU General Public
 * License Version 2
 */

#include "vl.h"
#include "s3c2410x.h"

/* Each BAST IDE region is 0x1000000 bytes long, 
 * the second half is the "alternate" register set 
 */

static void stcb_ide_write (void *opaque,
                            target_phys_addr_t addr, uint32_t val)
{
  int reg = (addr & 0x3ff) >> 5; /* 0x200 long, 0x20 stride */
  if( reg == 0 ) {
    /* Data register */
    ide_data_writew(opaque, 0, val);
  } else {
    /* Everything else */
    ide_ioport_write(opaque, reg, val);
  }
}

static uint32_t stcb_ide_read (void *opaque,target_phys_addr_t addr)
{
  int reg = (addr & 0x3ff) >> 5; /* 0x200 long, 0x20 stride */
  int alt = (addr & 0x800000) != 0;
  if( alt ) {
    return ide_status_read(opaque, 0);
  }
  if( reg == 0 ) {
    return ide_data_readw(opaque, 0);
  } else {
    return ide_ioport_read(opaque, addr);
  }
}


static CPUWriteMemoryFunc *stcb_ide_write[] = {
    stcb_ide_write,
    stcb_ide_write,
    stcb_ide_write,
};

static CPUReadMemoryFunc *stcb_ide_read[] = {
    stcb_ide_read,
    stcb_ide_read,
    stcb_ide_read,
};

/* hd_table must contain 2 block drivers */
/* BAST uses memory mapped registers, not I/O. Return the memory
   I/O index to access the ide. */
int stcb_ide_init (BlockDriverState **hd_table,
                   SetIRQFunc *set_irq, void *irq_opaque, int irq)
{
    IDEState *ide_if;
    int stcb_ide_memory;

    ide_if = qemu_mallocz(sizeof(IDEState) * 2);
    ide_init2(&ide_if[0], hd_table[0], hd_table[1],
              set_irq, irq_opaque, irq);
    
    stcb_ide_memory = cpu_register_io_memory(0, stcb_ide_read,
                                             stcb_ide_write, &ide_if[0]);
    return stcb_ide_memory;
}
